package com.ep26.chatapp;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.widget.Toast;

public class ChatService extends Service {

	NotificationManager mNotificationManager;
	UDPServerThread mServerThread = new UDPServerThread();

	final Handler mServiceHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {

			MessageToService msgToService = (MessageToService) msg.obj;
			String remoteIP = msgToService.remoteIP;
			String remotePort = msgToService.remotePort;
			String receivedString = msgToService.receivedString;
			String name = msgToService.name;

			// create a notification for a missed message
			int icon = R.drawable.chatblack;
			CharSequence tickerText = receivedString;
			long when = System.currentTimeMillis();

			Notification notification = new Notification(icon, tickerText, when);
			Context context = getApplicationContext();
			CharSequence contentTitle = "Message from " + name;
			CharSequence contentText = receivedString;

			Intent i = new Intent(getApplicationContext(),
					SimpleMessenger.class);
			i.putExtra("ipAddress", remoteIP);
			i.putExtra("port", remotePort);
			i.putExtra("name", name);
			PendingIntent contentIntent = PendingIntent.getActivity(context, 0,
					i, 0);

			notification.defaults |= Notification.DEFAULT_SOUND;
			notification.defaults |= Notification.DEFAULT_LIGHTS;
			// make the notification disappear after clicking on it
			notification.flags |= Notification.FLAG_AUTO_CANCEL;

			notification.setLatestEventInfo(context, contentTitle, contentText,
					contentIntent);
			mNotificationManager.notify(MISSED_CHAT_ID, notification);

			// add the chat to contact's chat history so that it doesn't get
			// lost
			ContactsActivity.addConversationHistoryToDb(remoteIP, name + ": "
					+ receivedString + "\n");

		}
	};

	// unique identification numbers for notifications
	private static final int MISSED_CHAT_ID = 1;

	@Override
	public void onCreate() {
		mServerThread.serviceHandler = this.mServiceHandler;
		// Start up the thread running the service. Note that we create a
		// separate thread because the service normally runs in the process's
		// main thread, which we don't want to block.
		if (!mServerThread.isAlive()) {
			mServerThread.start();
		}
		// Create the notification manager
		mNotificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		// Toast.makeText(this, "service starting", Toast.LENGTH_SHORT).show();
		// If we get killed, after returning from here, restart
		return START_STICKY;
	}

	@Override
	public IBinder onBind(Intent intent) {
		// We don't provide binding, so return null
		return null;
	}

	@Override
	public void onDestroy() {
		Toast.makeText(this, "service destroyed", Toast.LENGTH_SHORT).show();
	}

}