package com.ep26.chatapp;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.provider.BaseColumns;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnCreateContextMenuListener;
import android.view.View.OnLongClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Toast;

public class ContactsActivity extends Activity {

	private static ContactDataSQLHelper eventsData;
	private static ListView mContactListView;
	private static Cursor cursor;
	public static ContactsAdapter mContactsArrayAdapter;
	public static ListView mConversationView;

	protected static final int CONTEXTMENU_DELETEITEM = 0;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Starting the chat service
		Intent chatServiceIntent = new Intent(this, ChatService.class);
		startService(chatServiceIntent);

		setContentView(R.layout.contacts);
		// Creating the database helper
		eventsData = new ContactDataSQLHelper(this);

		// Initialize the array adapter
		mContactsArrayAdapter = new ContactsAdapter(this, R.layout.contact);
		// Initialize the ListView
		mContactListView = (ListView) findViewById(R.id.contacts_list);
		mContactListView.setAdapter(mContactsArrayAdapter);
		// //!!!droping the whole table
		// emptyContactsDbTable();

		// Populate the ListView by reading the database
		cursor = getContacts();
		populateListViewFromDb(cursor);
		registerForContextMenu(mContactListView);

		// Initialise the 'Add new contact' button and set OnClickListener
		Button newContactButton = (Button) findViewById(R.id.newcontact);
		newContactButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				getNewContact();
			}
		});

		/* set the context menu on long-press for list items */
		mContactListView
				.setOnCreateContextMenuListener(new OnCreateContextMenuListener() {

					@Override
					public void onCreateContextMenu(ContextMenu menu, View v,
							ContextMenuInfo menuInfo) {
						menu.setHeaderTitle("Options");
						menu.add(0, CONTEXTMENU_DELETEITEM, 0, "Delete");
					}
				});

		/* set the long-press click listener for the list items (contacts) */
		mContactListView.setOnLongClickListener(new OnLongClickListener() {

			@Override
			public boolean onLongClick(View v) {
				// TODO Auto-generated method stub

				return false;
			}
		});

		/* set the click listener for list items (contacts) */
		mContactListView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {

				((ImageView) view.findViewById(R.id.ImageView01))
						.setImageResource((R.drawable.bx_chat));

				// When clicked, start a new SimpleMessenger activity
				Intent i = new Intent(getApplicationContext(),
						SimpleMessenger.class);
				// Passing the ipAddress to the new activity
				i.putExtra("ipAddress",
						mContactsArrayAdapter.getItem(position).address);
				i.putExtra("port", mContactsArrayAdapter.getItem(position).port);
				i.putExtra("name", mContactsArrayAdapter.getItem(position).name);
				startActivity(i);
			}
		});
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		eventsData.close();
	}

	// the context menu that appears after long pressing a contact
	public boolean onContextItemSelected(MenuItem aItem) {
		AdapterContextMenuInfo menuInfo = (AdapterContextMenuInfo) aItem
				.getMenuInfo();
		/* Switch on the ID of the item, to get what the user selected. */
		switch (aItem.getItemId()) {
		case CONTEXTMENU_DELETEITEM:
			/* Get the selected contact out of the Adapter by its position. */
			Contact contactSelected = (Contact) mContactListView.getAdapter()
					.getItem(menuInfo.position);
			// remove the contact from the listview
			mContactsArrayAdapter.remove(contactSelected);

			// remove the contact from the database
			removeContactFromDb(contactSelected);
			return true;
		}
		return false;
	}

	/* Adds a Contact object to the ListView */
	private static void addContactToListView(Contact newContact) {
		try {
			if (newContact != null) {
				mContactsArrayAdapter.add(newContact);
			}

		} catch (Exception e) {
			Log.d("addContactToListView", "on adding", e.getCause());
		}
	}

	/* Displays an alert dialog in order to get input for creating a new contact */
	private void getNewContact() {
		final AlertDialog.Builder alert = new AlertDialog.Builder(this);
		LayoutInflater factory = LayoutInflater.from(this);
		final View textEntryView = factory.inflate(
				R.layout.dialog_contact_entry, null);
		alert.setView(textEntryView);

		alert.setTitle("Add contact");
		alert.setMessage("Enter contact's details:");

		final EditText nameInput = (EditText) textEntryView
				.findViewById(R.id.contact_name_dialog);
		final EditText addressInput = (EditText) textEntryView
				.findViewById(R.id.contact_ip_dialog);
		final EditText portInput = (EditText) textEntryView
				.findViewById(R.id.contact_port_dialog);

		// Set the action for the Ok Dialog box
		alert.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {

				String newContactName = nameInput.getText().toString().trim();
				String newContactIP = addressInput.getText().toString().trim();
				String newContactPort = portInput.getText().toString().trim();

				Contact contact = new Contact(newContactName, newContactIP,
						newContactPort);
				// add contact to db and listview unless it already exists
				try {
					if (newContactIP.equals("") || newContactName.equals("")
							|| newContactPort.equals("")) {
						Toast.makeText(getApplicationContext(),
								"The fields can't be empty!", Toast.LENGTH_LONG)
								.show();
					} else {
						addContact(contact);

					}
				} catch (Exception e) {
					Toast.makeText(getApplicationContext(),
							"This IP already exists.", Toast.LENGTH_LONG)
							.show();

				}
			}
		});

		// Set the action for the Cancel Dialog box
		alert.setNegativeButton("Cancel",
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						// Canceled.
						dialog.cancel();
					}
				});
		alert.show();
	}

	public static void addContact(Contact contact) {

		if (addContactToDb(contact) != -1)
			addContactToListView(contact);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.contacts_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle item selection
		switch (item.getItemId()) {

		case R.id.quit:
			quit();
			return true;
		case R.id.whatismyip: {
			try {
				URL whatismyip = new URL(
						"http://www.whatismyip.com/automation/n09230945.asp");
				BufferedReader in = new BufferedReader(new InputStreamReader(
						whatismyip.openStream()));
				String myIp = in.readLine();
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				builder.setMessage(
						"Your IP is " + myIp + "\nYour port is "
								+ UDPServerThread.port).setPositiveButton(
						"Thanks",
						new android.content.DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int id) {
								dialog.cancel();

							}
						});
				AlertDialog alert = builder.create();
				alert.show();

			} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

		default:
			return super.onOptionsItemSelected(item);
		}
	}

	/*
	 * When the user quits we just move the app to the background, android
	 * handles the destroying of tasks
	 */
	private void quit() {
		this.moveTaskToBack(true);
	}

	/*
	 * Populates the contacts listview with the contacts it read from the
	 * database earlier
	 */
	private void populateListViewFromDb(Cursor cursor) {
		while (cursor.moveToNext()) {
			String name = cursor.getString(0);
			String ipaddr = cursor.getString(1);
			String port = cursor.getString(2);

			addContactToListView(new Contact(name, ipaddr, port));
		}
	}

	/*--------------------------------DB METHODS---------------------------*/
	/* read all contacts from the database in order to create our list */
	private Cursor getContacts() {
		SQLiteDatabase db = eventsData.getReadableDatabase();
		cursor = db.query(ContactDataSQLHelper.TABLE, new String[] {
				ContactDataSQLHelper.NAME, ContactDataSQLHelper.IPADDR,
				ContactDataSQLHelper.PORT }, null, null, null, null, null);

		startManagingCursor(cursor);
		return cursor;
	}

	/*
	 * Adds a contact to the database. Returns the row ID of the newly inserted
	 * row, or -1 if an error occurred
	 */
	private static long addContactToDb(Contact contact) {
		SQLiteDatabase db = eventsData.getWritableDatabase();
		ContentValues values = new ContentValues();
		values.put(ContactDataSQLHelper.NAME, contact.name);
		values.put(ContactDataSQLHelper.IPADDR, contact.address);
		values.put(ContactDataSQLHelper.PORT, contact.port);

		return db.insert(ContactDataSQLHelper.TABLE, null, values);
	}

	/* Removes a contact from the database. Returns number of rows deleted */
	private int removeContactFromDb(Contact contact) {
		try {
			SQLiteDatabase db = eventsData.getReadableDatabase();
			return db.delete(ContactDataSQLHelper.TABLE,
					ContactDataSQLHelper.IPADDR + "=?",
					new String[] { contact.address });
		} catch (Exception e) {
			Log.d("removeContactFromDb", "on deleting", e.getCause());
		}
		return 0;
	}

	/*
	 * returns the conversation history from the db, given the ip. The
	 * conversation is stored as a single String using \n as a delimiter between
	 * messages
	 */
	public static String getConversationHistoryFromDb(String ipaddress) {

		SQLiteDatabase db = eventsData.getReadableDatabase();
		String query = "select " + ContactDataSQLHelper.HISTORY + " from "
				+ ContactDataSQLHelper.TABLE + " where "
				+ ContactDataSQLHelper.IPADDR + "= ?";
		try {
			cursor = db.rawQuery(query, new String[] { ipaddress });
			if (cursor.moveToFirst()) {
				return cursor.getString(0);
			}
		} catch (Exception e) {
			Log.d("addConversationHistoryToDb", "on adding", e.getCause());
		}

		return null;
	}

	/*
	 * Saves the conversation history to the database. returns 1 if succeeded 0
	 * otherwise
	 */
	public static int addConversationHistoryToDb(String ipaddress,
			String history) {
		long id;
		ContentValues values = new ContentValues();
		String previousHistory = getConversationHistoryFromDb(ipaddress);
		if (previousHistory != null)
			history = previousHistory + history;

		SQLiteDatabase db = eventsData.getWritableDatabase();
		String query = "select " + BaseColumns._ID + " from "
				+ ContactDataSQLHelper.TABLE + " where "
				+ ContactDataSQLHelper.IPADDR + "= ?";
		try {
			cursor = db.rawQuery(query, new String[] { ipaddress });
			if (cursor.moveToFirst()) {
				id = cursor.getLong(0);
				values.put(ContactDataSQLHelper.HISTORY, history);
				return db.update(ContactDataSQLHelper.TABLE, values,
						BaseColumns._ID + "=" + id, null);
			}
		} catch (Exception e) {
			Log.d("addConversationHistoryToDb", "on adding", e.getCause());
		}
		return 0;
	}

	/*
	 * Returns the contact's name given their IP from the database, if it
	 * doesn't exist return null.
	 */
	public static String getNameFromDb(String ipaddress) {

		SQLiteDatabase db = eventsData.getReadableDatabase();
		try {
			cursor = db.query(ContactDataSQLHelper.TABLE,
					new String[] { ContactDataSQLHelper.NAME },
					ContactDataSQLHelper.IPADDR + "= ?",
					new String[] { ipaddress }, null, null, null);
			if (cursor.moveToFirst())
				return cursor.getString(0);
		} catch (Exception e) {
			Log.d("getNameFromDb", "on query", e.getCause());

		}
		return null;
	}

	// Drops the whole table, don't call
	private void emptyContactsDbTable() {
		SQLiteDatabase db = eventsData.getWritableDatabase();
		eventsData.onUpgrade(db, 1, 2);
	}

	/*--------------------------------END OF DB METHODS---------------------------*/

}