package com.ep26.chatapp;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.UnknownHostException;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Html;
import android.text.InputType;
import android.text.Spanned;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnKeyListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class SimpleMessenger extends Activity {
	/** Called when the activity is first created. */

	protected String contactIP;
	protected String contactName;
	private InetAddress contactInetAddress;
	private String mReceivedMessage;
	private String history = "";

	private ListView mConversationView;
	private Button mSendButton;
	private EditText mTextInput;
	private TextView mWarningText;
	private ArrayAdapter<Spanned> mConversationArrayAdapter;

	private int contactPort = UDPServerThread.port;

	final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			// display the message in the conversation list
			mReceivedMessage = (String) msg.obj;
			updateMessagesInUi(contactName, mReceivedMessage);
			// clear the text area of the previous message
			mTextInput.setText("");
		}
	};

	@Override
	protected void onPause() {
		super.onPause(); // the UI component values are saved here.
		if (history != null) {
			ContactsActivity.addConversationHistoryToDb(contactIP, history);
		}
		UDPServerThread.setHandler(null);

	}

	@Override
	// todo: add option to delete history on chat menu
	// add option to copy message text on long press
	protected void onResume() {
		super.onResume(); // the UI component values are saved here.
		UDPServerThread.setRemoteAddress(contactIP);
		String history = ContactsActivity
				.getConversationHistoryFromDb(contactIP);
		if (history != null) {
			String[] lines = history.split("\n");
			for (String line : lines) {
				mConversationArrayAdapter
						.add(Html.fromHtml("<font color=\"#999999\">" + line
								+ "</font>"));
			}
		}
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		UDPServerThread.setHandler(mHandler);
		setContentView(R.layout.chat_window);

		this.contactIP = getIntent().getStringExtra("ipAddress");

		this.contactPort = Integer.parseInt(getIntent().getStringExtra("port"));
		this.contactName = getIntent().getStringExtra("name");
		UDPServerThread.setRemoteAddress(contactIP);

		// Initialising the button and EditTexts
		mSendButton = (Button) findViewById(R.id.sendButton);
		mTextInput = (EditText) findViewById(R.id.messageText);
		mTextInput.requestFocus();
		mTextInput.setOnKeyListener(new OnKeyListener() {
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				// If the event is a key-down event on the "enter" button
				if ((event.getAction() == KeyEvent.ACTION_DOWN)
						&& (keyCode == KeyEvent.KEYCODE_ENTER)) {
					// Perform action on key press
					sendAMessage();
					return true;
				}
				return false;
			}
		});
		mWarningText = (TextView) findViewById(R.id.chat_warning);
		// reachability check
		try {
			this.contactInetAddress = InetAddress.getByName(contactIP);
			if (!contactInetAddress.isReachable(300)) {
				mWarningText
						.setText("This contact seems to be offline. Your messages might get lost.");
				mWarningText.setTextColor(Color.parseColor("#C40031"));
				mWarningText.setBackgroundColor(Color.parseColor("#FFE6EC"));
			}
			// setOffline(true);
		} catch (UnknownHostException e) {
			setOffline(true);
		} catch (IOException e) {
			e.printStackTrace();
		}

		// Initialising the Array Adapter and the list view
		mConversationArrayAdapter = new ArrayAdapter<Spanned>(this,
				R.layout.message);
		mConversationView = (ListView) findViewById(R.id.in);
		mConversationView.setAdapter(mConversationArrayAdapter);

		// Initialise the send button with a listener for click events
		mSendButton.setOnClickListener(new sender());
		this.setTitle("Chat with " + contactName);

	}

	private void setOffline(boolean b) {
		if (b) {
			// set offline
			Toast.makeText(getApplicationContext(), "This seems to be an invalid IP",
					Toast.LENGTH_SHORT).show();

			this.mTextInput.setEnabled(false);
		} else {
			// set online
			this.mTextInput.setEnabled(true);

		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.chat_menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {

		case R.id.quit:
			quit();
			return true;
		case R.id.switch_chats:
			Intent intent = new Intent();
			setResult(RESULT_OK, intent);
			finish();
			return true;
		default:
			// return super.onOptionsItemSelected(item);
			return true;
		}
	}

	private void sendAMessage() {

		String message = mTextInput.getText().toString().trim();
		if (message.equals(""))
			return;

		try {
			setOffline(false);
			mTextInput.setInputType(InputType.TYPE_TEXT_FLAG_AUTO_COMPLETE);
			/* a string to be sent */
			byte[] sendBuf = (message).getBytes();

			/* create a datagram */
			DatagramPacket pkt = new DatagramPacket(sendBuf, sendBuf.length,
					contactInetAddress, contactPort);

			/* create a socket */
			DatagramSocket skt = new DatagramSocket();

			/* send the datagram */
			skt.send(pkt);

			// display it in the conversation view
			updateMessagesInUi("me", message);
			// clear the EditText message area
			mTextInput.setText("");

			/* close socket */
			skt.close();
		} catch (Exception e) {
			Toast.makeText(getApplicationContext(),
					"Something went wrong with sending the message",
					Toast.LENGTH_SHORT).show();
		}
	}

	class sender implements Button.OnClickListener {

		public void onClick(View v) {
			sendAMessage();
		}
	}

	private void quit() {
		this.moveTaskToBack(true);
	}

	// writes a message to the conversation listview
	protected void updateMessagesInUi(String who, String message) {

		Spanned str = Html.fromHtml("<b>" + who + ": " + "</b>" + message);
		mConversationArrayAdapter.add(str);
		history += str.toString() + "\n";
	}

}